/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.reporting;

import static org.assertj.core.api.Assertions.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import de.ozgcloud.admin.common.user.UserRole;
import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.common.test.TestUtils;
import lombok.SneakyThrows;

@AutoConfigureMockMvc
@WithMockUser(roles = UserRole.DATENBEAUFTRAGUNG)
@DataITCase
class ReportingSettingITCase {

	@Autowired
	private MockMvc mockMvc;

	@Autowired
	private MongoOperations mongoOperations;

	@BeforeEach
	void clearDatabase() {
		mongoOperations.dropCollection("settings");
	}

	@Test
	@SneakyThrows
	void shouldHaveLinkToAggregationMapping() {
		mockMvc.perform(get("/api/configuration"))
				.andExpect(status().is2xxSuccessful())
				.andExpect(jsonPath("$._links.aggregationMappings").exists());
	}

	@Test
	@SneakyThrows
	@WithMockUser(roles = UserRole.ADMIN_ADMIN)
	void shouldNotHaveLinkOnMissingRole() {
		mockMvc.perform(get("/api/configuration"))
				.andExpect(status().is2xxSuccessful())
				.andExpect(jsonPath("$._links.aggregationMappings").doesNotExist());
	}

	@Test
	@SneakyThrows
	void shouldDenyWhileMissingFields() {
		mockMvc.perform(post("/api/configuration/aggregationMappings").with(csrf())
				.contentType(MediaType.APPLICATION_JSON).content("{}"))
				.andExpect(status().isUnprocessableEntity());
	}

	@Disabled("is returning 500")
	@Test
	@SneakyThrows
	@WithMockUser(roles = UserRole.ADMIN_ADMIN)
	void shouldNotAllowAddingMapping() {
		mockMvc.perform(post("/api/configuration/aggregationMappings").with(csrf())
				.contentType(MediaType.APPLICATION_JSON).content(TestUtils.loadTextFile("reporting/request.json")))
				.andExpect(status().is(403));
	}

	@Test
	@SneakyThrows
	void shouldAddAggregationMappings() {
		mockMvc.perform(post("/api/configuration/aggregationMappings").with(csrf())
				.contentType(MediaType.APPLICATION_JSON).content(TestUtils.loadTextFile("reporting/request.json")))
				.andExpect(status().isCreated());

		var collection = mongoOperations.getCollection("settings");
		assertThat(collection.countDocuments()).isEqualTo(1);
		var mapping = mongoOperations.findAll(AggregationMapping.class).getFirst();
		assertThat(mapping).usingRecursiveComparison().ignoringFields("id").isEqualTo(AggregationMappingTestFactory.create());
	}

	@Test
	@SneakyThrows
	void shouldListReportings() {
		mockMvc.perform(get("/api/configuration/aggregationMappings"))
				.andExpect(status().is2xxSuccessful());
	}

}
